/* Mtest - returns the observed M statistic and the p-value (Monte Carlo resampling) of M under the null hypothesis that Cases and Controls follow the same spatial distribution
Pietro Tebaldi , Harvard School of Public Health and Bocconi University
July 2010 */

program define Mtest
version 10.1
syntax , x(varname) y(varname) g(varname) [ bins(integer 20) iter(integer 100) level(real 95) SCatter DENsity ]
tempname Sinv d
if "`scatter'" == "scatter" {
tempvar X1 X2 Y1 Y2
quietly {
gen `X1' = `x' if `g'==0
gen `Y1' = `y' if `g'==0
gen `X2' = `x' if `g'==1
gen `Y2' = `y' if `g'==1
label var `X1' "Group 2"
label var `Y1' "Group 2"
label var `X2' "Group 1"
label var `Y2' "Group 1"
}
capture scatter `Y1' `X1'  , msymbol(oh) mcolor(black) ytitle("y") xtitle("x") title("Spatial Distribution of the two groups") || scatter `Y2' `X2' , msymbol(x) mcolor(black) , nodraw saving(scatter)
}
if "`density'" == "density" {
preserve
capture count
local N = r(N)
local Ndist = comb(`N',2)
quietly {
gen X1 = `x' if `g'==0
gen Y1 = `y' if `g'==0
label var X1 "Group 2"
label var Y1 "Group 2"
drop if X1 == .
eucldist X1 Y1
mata: g0 = st_data(.,1)							
clear
restore
preserve
gen X2 = `x' if `g'==1
gen Y2 = `y' if `g'==1
label var X2 "Group 1"
label var Y2 "Group 1"
drop if X2 == .
eucldist X2 Y2
mata: g1 = st_data(.,1)																
clear
}
mata: Nca = rows(g1)
mata: Nco = rows(g0)
quietly {
set obs `Ndist'
mata: ca = st_addvar("float","Group1")
mata: co = st_addvar("float","Group2")
mata: st_store((1,Nca),ca,g1)
mata: st_store((1,Nco),co,g0)
kdensity Group1 , gen(x1 d1) nograph
kdensity Group2 , gen(x2 d2) nograph
line d1 x1 , lcolor(black) lpattern(dash) ytitle("density") xtitle("interpoint distance") title("Kernel Densities") || line d2 x2 , lcolor(black) lpattern(solid)   , nodraw saving(density)
clear
restore
}
}
if "`scatter'" == "scatter" {
graph use scatter.gph
erase scatter.gph
}
if "`density'" == "density" {
graph use density.gph
erase density.gph
}
capture Mstat , x(`x') y(`y') g(`g') bins(`bins')
matrix `d' = r(d)
matrix `Sinv' = r(Sinv)
display "{txt}{hline 77}"
display as text "M statistic"
display as text "Monte Carlo permutation results"
display as text "H0: The two groups have the same spatial distribution"
display as text "Number of bins = `bins'"
display as text "Number of permutations = `iter'"
permute `g' M=r(M) , reps(`iter') right level(`level') nodots nowarn noheader : Mst_iter , x(`x') y(`y') g(`g') sinv(`Sinv') d(`d')
end

